// Battery powered control from dining wall to cloackroom controller
// V 2 Push button applies power, sends single toggle command and goes to sleep

#include <Arduino.h>
#include <ESP8266WiFi.h>
extern "C" {
#include <espnow.h>
#include <user_interface.h>
}

#define WIFI_DEFAULT_CHANNEL 10


uint8_t no1[] = {0x2E, 0X3A, 0XE8, 0X32, 0X7D, 0XC9};   // in-line vertical board 1
uint8_t no2[] = {0x2E, 0X3A, 0XE8, 0X32, 0XB1, 0X3D};   // in-line vertical board 2
uint8_t no3[] = {0x62, 0x01, 0x94, 0x2A, 0x10, 0x16};   // ESP1 module in X-10 hosing  dev #2

void printMacAddress(uint8_t* macaddr) 
{
  Serial.print("{");
  for (int i = 0; i < 6; i++) 
  {
    Serial.print("0x");
    Serial.print(macaddr[i], HEX);
    if (i < 5) Serial.print(',');
  }
  Serial.println("}");
}


void setup() 
{
  WiFi.disconnect();

  Serial.begin(115200);
  Serial.println("Initializing... CONTROLLER");

  WiFi.mode(WIFI_STA);

  uint8_t macaddr[6];
  
  wifi_get_macaddr(STATION_IF, macaddr);
  Serial.print("mac address (STATION_IF): "); printMacAddress(macaddr);

  if (esp_now_init() == 0) 
  {
    Serial.println("init");
  } 
  else 
  {
    Serial.println("init failed");
    ESP.restart();
    return;
  }

  Serial.println("ESP_NOW_ROLE_CONTROLLER");
  
  esp_now_set_self_role(ESP_NOW_ROLE_CONTROLLER);
  
  esp_now_register_recv_cb([](uint8_t *macaddr, uint8_t *data, uint8_t len) {
    Serial.println("recv_cb: mac address: ");
    printMacAddress(macaddr);
    Serial.println();

    Serial.print("data: ");
    for (int i = 0; i < len; i++) 
    {
      Serial.print(data[i], HEX);
    }
    Serial.print("  Addr:");

    for (int i = 0; i < 6; i++) 
    {
      Serial.print(macaddr[i], HEX);
      if (i < 5) Serial.print(',');
    }
    Serial.println("");
  });

  esp_now_register_send_cb([](uint8_t* macaddr, uint8_t status) {
    Serial.print("send_cb: ");
    Serial.print("mac address: ");
    printMacAddress(macaddr);
    Serial.print(" status = "); Serial.println(status);
  });

  int res1 = esp_now_add_peer(no1, (uint8_t)ESP_NOW_ROLE_SLAVE, 
        (uint8_t)WIFI_DEFAULT_CHANNEL, NULL, 0);
  int res2 = esp_now_add_peer(no2, (uint8_t)ESP_NOW_ROLE_SLAVE, 
        (uint8_t)WIFI_DEFAULT_CHANNEL, NULL, 0);
}

uint8_t message[] = { 'b', 'l', 'h', 'A', '2', '\0' };   // #9 toggle

void loop() 
{
//  uint8_t bs[sizeof(message)];
//  memcpy(bs, &message, sizeof(message));

  esp_now_send( no1, &message[0], sizeof(message) );    
  esp_now_send( no2, &message[0], sizeof(message) );    
//  esp_now_send( NULL, &message[0], sizeof(message) );    

  Serial.print("Data to slave:");
  Serial.write(message[0]);
  Serial.write(message[1]);
  Serial.write(message[2]);
  Serial.write(message[3]);
  Serial.write(message[4]);
  Serial.println("");

  ESP.deepSleep( 1e6,  WAKE_RF_DEFAULT );   // 15mA + 300mA peak
  for(;;) yield();  
}

