/* 
  Front door computer:
    1. Turn on dining lights from ESP8266 button by kitchen
    2. Turn on dining lights by button D on this computer
    3. Toggle corner outside light via EPSnow link
    4. Turn on porch lights
    5. Time control 3+4
    6. Entry lighting from front door switch

    1A. Device code 9, work in toggle mode for remote and local button

    Control Message:
    'blh'
    dev code
    0 = off, 1 =  on, 2 = toggle
*/

#include <Wire.h>  
#include "SSD1306.h"      // alias for `#include "SSD1306Wire.h"`

//SSD1306  display(0x3c, D2, D1);
SSD1306  display( 0x3c, 4, 5 );   // sda, scl


#include <Arduino.h>
#include <ESP8266WiFi.h>
extern "C" {
#include <espnow.h>
#include <user_interface.h>
}

#define WIFI_DEFAULT_CHANNEL 10

#define relayP 12       // corner light
#define relayD 13       // dining lights

#define buttonD 16      // dining lights
#define buttonP 14      // outside corner light

//uint8_t no1[] = {0x2C, 0x3A, 0xEB, 0x15, 0x33, 0x34};   // NodeMCU number 1
//uint8_t no2[] = {0x5C, 0xCF, 0x7F, 0xAB, 0xBD, 0x19};   // NodeMCU number 2

uint8_t Corner[] = {0x62, 0x01, 0x94, 0x51, 0xD1, 0x21};   // ESP01 module
#define DevCorner '8'
uint8_t message[] = { 'b', 'l', 'h', DevCorner, '0', '\0' };
int16_t secs = 0;
#define secdel 600     // seconds delay

void printMacAddress(uint8_t* macaddr) 
{
  Serial.print("{");
  for (int i = 0; i < 6; i++) 
  {
    Serial.print("0x");
    Serial.print(macaddr[i], HEX);
    if (i < 5) Serial.print(',');
  }
  Serial.print("}");
}


void updateScreen()
{
  String st = "Exit: "; 
  uint8_t blank = true;
  
  display.clear(); 

  if ( digitalRead( relayP ) == LOW )
  {
    display.drawString(0, 0, "Ext Lights"); 
    blank = false;
  }

  if ( digitalRead( relayD ) == LOW )
  {
    display.drawString(0, 20, "Dining"); 
    blank = false;
  }

  if ( secs > 0 )
  {
    st = st + String( secs ); 
    display.drawString( 0, 40, st );
    blank = false;
  }

  if ( blank ) 
  { 
    //display.setFont(ArialMT_Plain_24);
    display.drawString(0, 20, "   ** 65 **"); 
  }
 
  display.display();
}

void toggleP()
{
  if ( digitalRead( relayP ) == LOW )
  {
    Serial.println("P H");
    digitalWrite( relayP, HIGH );
    message[4] = '0';     // on
    esp_now_send( Corner, &message[0], sizeof(message) );
  }
  else
  {
    Serial.println("P L");
    digitalWrite( relayP, LOW );    
    
    message[4] = '1';     // off
    esp_now_send( Corner, &message[0], sizeof(message) );  
  }
}


void toggleD()
{
  if ( digitalRead( relayD ) == LOW )
  {
    Serial.println("D H");
    digitalWrite( relayD, HIGH );
  }
  else
  {
    Serial.println("D L");
    digitalWrite( relayD, LOW );    
  }
}


void buttonHandler()    // takes 1 sec
{
  if ( digitalRead( buttonP ) == LOW ) toggleP();  
  if ( digitalRead( buttonD ) == LOW ) toggleD(); 
  
  delay(500);

  if ( digitalRead( buttonP ) == LOW &&
    digitalRead( buttonD ) == LOW ) secs = secdel; // exit timer
  
  updateScreen();
  delay(500); 

  //while ( digitalRead( buttonP ) == LOW || digitalRead( buttonD ) == LOW ) delay(100);
}


void setup() 
{
  uint8_t macaddr[6];
  
  WiFi.disconnect();
  
  pinMode( LED_BUILTIN, OUTPUT );
  pinMode( relayP, OUTPUT );
  digitalWrite( relayP, HIGH );   // active low relay
  pinMode( relayD, OUTPUT );
  digitalWrite( relayD, HIGH );   // active low relay
  pinMode( buttonP, INPUT );
  pinMode( buttonD, INPUT );

  Serial.begin(115200);
  Serial.println("Initializing... SLAVE");

  WiFi.mode(WIFI_AP_STA);
  wifi_get_macaddr(SOFTAP_IF, macaddr);
  Serial.print("mac address (SOFTAP_IF): "); printMacAddress(macaddr);
  Serial.println();

  if (esp_now_init() == 0) 
  {
    Serial.println("Now init OK");
  } 
  else 
  {
    Serial.println("Now init failed");
    ESP.restart();
    return;
  }

  Serial.println("ESP_NOW_ROLE_SLAVE");
  esp_now_set_self_role(ESP_NOW_ROLE_SLAVE);
  
  esp_now_register_recv_cb([](uint8_t *macaddr, uint8_t *data, uint8_t len) 
  {                 // this code runs when data received
    Serial.print("Incoming: ");
    Serial.print("recv_cb: mac address:  "); printMacAddress(macaddr);
    Serial.print("  Len:"); Serial.println(len);
    Serial.print("data: ");
    for (int i = 0; i < len; i++) 
    {
      Serial.print(char(data[i]));
    }
    Serial.println();

    // 01234
    // blh90
    if ( data[0] == 'b' && data[1] == 'l' && data[2] == 'h' )     // it's legit
      if ( data[3] == '9' )   toggleD();         // dev 9 is toggle only

 // because this fires off at unknown time +2 ensures secs == secdel wiil be seen in loop()
     if ( data[0] == 'b' && data[1] == 'l' && data[2] == 'h' )     // it's legit
      if ( data[3] == 'A' )   secs = secdel+2;         // dev A is car button
  });


  // Initialising the UI will init the display too.
  display.init();
  display.flipScreenVertically();
  display.setFont(ArialMT_Plain_24);
  display.setTextAlignment(TEXT_ALIGN_LEFT);
  display.drawString(0, 20, "   ** 65 **"); 
  display.display();
}


void loop() 
{
  Serial.println("loop()");
  buttonHandler();

  if ( secs == secdel )   // init timer
  {
    digitalWrite( relayD, LOW );  
    digitalWrite( relayP, LOW );
    message[4] = '1';     // on
    esp_now_send( Corner, &message[0], sizeof(message) );
    secs--;
  }

  if ( secs > 0 )
  {
    secs--;           // assuming buttonHandler takes 1 sec
    updateScreen();
    if ( digitalRead( buttonP ) == LOW ) secs = 10;
    while ( digitalRead( buttonP ) == LOW ) delay(100);   // prevent passing to bHandler
    
    if ( secs < 1 )
    {
      digitalWrite( relayD, HIGH );  
      digitalWrite( relayP, HIGH );
      message[4] = '0';     // off
      esp_now_send( Corner, &message[0], sizeof(message) );
    }
  }
}

