/*  1B is for Mini D1 ( not pro )
 *  Nextion version. Touch screen replaces buttons and display
  Front door computer:
    1. Turn on dining lights from ESP8266 button by kitchen
    2. Turn on dining lights by button D on this computer
    3. Toggle corner outside light via EPSnow link
    4. Turn on porch lights
    5. Time control 3+4
    6. Entry lighting from front door switch

    1A. Device code 9, work in toggle mode for remote and local button

    Control Message:
    'blh
    dev code
    0 = off, 1 =  on, 2 = toggle
*/

#define dbug true
#define dbug2 true
const int Cred = 63488;
const int Cgreen = 2016;

#include <SoftwareSerial.h>


#define relayP 12       // D6 porch lights
#define relayD 13       // D7 dining lights
SoftwareSerial swSer(14, 16, false, 256);

uint8_t smsbuffer[16];
uint8_t dOn = false;        // the state of 3 switched circuits
uint8_t cOn = false;        // doStates actions what needs doing 
uint8_t pOn = false;        // based on these
uint8_t darkTimer = false;  // entry exit timer
String textN;
unsigned long aSecond;  // for non block seconds timer
int16_t Tsecs = 0;
int16_t lastTsecs = 0;
#define secdel 600     // seconds delay

#include <Arduino.h>
#include <ESP8266WiFi.h>
extern "C" {
#include <espnow.h>
#include <user_interface.h>
}

#define WIFI_DEFAULT_CHANNEL 10

//uint8_t no1[] = {0x2C, 0x3A, 0xEB, 0x15, 0x33, 0x34};   // NodeMCU number 1
//uint8_t no2[] = {0x5C, 0xCF, 0x7F, 0xAB, 0xBD, 0x19};   // NodeMCU number 2

uint8_t Corner[] = {0x62, 0x01, 0x94, 0x51, 0xD1, 0x21};   // ESP01 module
#define DevCorner '8'
uint8_t message[] = { 'b', 'l', 'h', DevCorner, '0', '\0' };

void printMacAddress(uint8_t* macaddr) 
{
  Serial.print("{");
  for (int i = 0; i < 6; i++) 
  {
    Serial.print("0x");
    Serial.print(macaddr[i], HEX);
    if (i < 5) Serial.print(',');
  }
  Serial.print("}");
}


void toNextion( String  s )
{
  swSer.print( s );
  swSer.write(255);
  swSer.write(255);
  swSer.write(255);
}


void doStates()   // update switches and screen
{
  if ( dOn )
  {
    toNextion("b0.bco=63488");    // red
    digitalWrite( relayD, LOW );  // relay on
  }
  else
  {
    toNextion("b0.bco=2016");     // green
    digitalWrite( relayD, HIGH );
  }   

  if ( cOn )
  {
    toNextion("b1.bco=63488");
    message[4] = '1';     // on
    esp_now_send( Corner, &message[0], sizeof(message) );
  }
  else
  {
    toNextion("b1.bco=2016");
    message[4] = '0';     // off
    esp_now_send( Corner, &message[0], sizeof(message) );
  } 
    
  if ( pOn )
  {
    toNextion("b2.bco=63488");
    digitalWrite( relayP, LOW );  
  }
  else
  {
    toNextion("b2.bco=2016");
    digitalWrite( relayP, HIGH );
  }   

  textN = "t0.txt=\"";              // update banner
  if ( dOn ) textN.concat("Din");
  if ( pOn ) textN.concat(" Por");
  if ( cOn ) textN.concat(" Cor");
  if ( !dOn && !pOn && !cOn ) textN.concat("** 65 **"); // idle message
  textN.concat( "\"" );
  toNextion( textN );
}

// Nextion comms ========================
/* Handle incoming nextion data
 * b0 = ID 0001 Dining 
 * b1 = ID 0002 Corner
 * b2 = ID 0003 Porch 
 * b3 = ID 0005 All
 * b4 = ID 0006 Timed
 * b6 = ID 0007 Off
 */

uint8_t intosmsbuffer()
{
  uint8_t button;

  // looking for e 0000 00 0x255 0x255 ox255   9ch
  //               bbbb 01    button 00 or 01

  smsbuffer[0] = smsbuffer[1];  // keep adding to end of array
  smsbuffer[1] = smsbuffer[2];
  smsbuffer[2] = smsbuffer[3];
  smsbuffer[3] = smsbuffer[4];
  smsbuffer[4] = smsbuffer[5];
  smsbuffer[5] = smsbuffer[6];
  smsbuffer[6] = swSer.read();  // new char onto tail

  if ( dbug )
  {
    Serial.print("SMSbuffer=");
    Serial.print( smsbuffer[0] ); Serial.print(",");
    Serial.print( smsbuffer[1] ); Serial.print(",");
    Serial.print( smsbuffer[2] ); Serial.print(",");
    Serial.print( smsbuffer[3] ); Serial.print(",");
    Serial.print( smsbuffer[4] ); Serial.print(",");
    Serial.print( smsbuffer[5] ); Serial.print(",");
    Serial.println( smsbuffer[6] );
  }

  button = 0;
  
  if( (smsbuffer[0] == 101)       // "e"
        && (smsbuffer[4] == 255)
        && (smsbuffer[5] == 255)
        && (smsbuffer[6] == 255))
    {
      button = smsbuffer[2];
      // bstate = smsbuffer[3];

      if ( button == 1 )     // Dining
        if ( dOn ) dOn = false;
        else dOn = true;
      
      if ( button == 2 )   // Corner
        if ( cOn ) cOn = false;
        else cOn = true;
           
      if ( button == 3 )   // Porch
        if ( pOn ) pOn = false;
        else pOn = true;

      if ( button == 5 )    // Exit
      {
        darkTimer = true;
        Tsecs = secdel;
        dOn = true;
        pOn = true;
        cOn = true;
      }

      if ( button == 6 )    // turns everything off after timer
      {
        darkTimer = true;
        Tsecs = secdel;
      }
            
      if ( button == 7 )    // Off
      {
        dOn = false;
        pOn = false;
        cOn = false;
        darkTimer = false;
      }
    }

    if(dbug) Serial.print("Button:"); Serial.println(button);

    if ( button > 0 )     // got one, action it and clear buffer
    {
      doStates();
      smsbuffer[0] = 0;
      smsbuffer[1] = 0;
      smsbuffer[2] = 0;
      smsbuffer[3] = 0;
      smsbuffer[4] = 0;
      smsbuffer[5] = 0;
      smsbuffer[6] = 0;
    }
}
//  Nextion comms ========================

// {0x2E, 0X3A, 0XE8, 0X32, 0X7D, 0XC9};   // in-line vertical board 1
void setup() 
{
  uint8_t macaddr[6];
  
  WiFi.disconnect();
  WiFi.mode(WIFI_AP_STA);       
  
  // Change MAC so other units do not need reprogramming 
  byte *mac = (byte*)malloc(6);
  WiFi.macAddress(mac);
  mac[0] = 0x2E;
  mac[1] = 0x3A;
  mac[2] = 0xE8;
  mac[3] = 0x32;
  mac[4] = 0x7D;
  mac[5] = 0xC9;
  wifi_set_macaddr(STATION_IF, mac);
//  mac[2] = 0xCB;
//  wifi_set_macaddr(SOFTAP_IF, mac);
  free(mac);


  
  pinMode( LED_BUILTIN, OUTPUT );
  pinMode( relayP, OUTPUT );
  digitalWrite( relayP, HIGH );   // active low relay
  pinMode( relayD, OUTPUT );
  digitalWrite( relayD, HIGH );   // active low relay
  
  swSer.begin(9600);    // nextion interface
  toNextion("t0.txt=\"Start Up V:Next1A\"");
  Serial.begin(115200);
  Serial.println("Initializing... SLAVE");
 
  //WiFi.mode(WIFI_AP_STA);
  wifi_get_macaddr(SOFTAP_IF, macaddr);
  Serial.print("mac address (SOFTAP_IF): "); printMacAddress(macaddr);
  Serial.println();

  if (esp_now_init() == 0) 
  {
    Serial.println("Now init OK");
  } 
  else 
  {
    Serial.println("Now init failed");
    ESP.restart();
    return;
  }

  Serial.println("ESP_NOW_ROLE_SLAVE");
  esp_now_set_self_role(ESP_NOW_ROLE_SLAVE);

  // this code runs when data received
  esp_now_register_recv_cb([](uint8_t *macaddr, uint8_t *data, uint8_t len) 
  {                 
    if( dbug2 )
    {
      Serial.print("Incoming: ");
      Serial.print("recv_cb: mac address:  "); printMacAddress(macaddr);
      Serial.print("  Len:"); Serial.println(len);
      Serial.print("data: ");
      for (int i = 0; i < len; i++) 
      {
        Serial.print(char(data[i]));
      }
      Serial.println();
    }

    // 01234
    // blh90  9 = Dining remote push
    if ( data[0] == 'b' && data[1] == 'l' && data[2] == 'h' )     // it's legit
    {
      if ( data[3] == '9' )       // toggleD();  // dev 9 is toggle only
      {
        if ( dOn ) dOn = false;
        else dOn = true;

        doStates();
      }

      if ( data[3] == 'A' )       // dev A is car button
      {
        darkTimer = true; 
        Tsecs = secdel; 
        dOn = true;
        pOn = true;
        cOn = true;
        doStates();
      }
    }
  });

  toNextion("b3.txt=\"Exit\"");   // make b3 the timed exit button
  aSecond = millis() + 1000;
}


void loop() 
{
  if( swSer.available() > 0 ) intosmsbuffer(); // log incoming from nextion

  
  if( aSecond <= millis() )    // 1 second sevices.
  {    
    aSecond = millis() + 1000;  // 1 second on

    // Serial.print("Tsecs: "); Serial.println( Tsecs );

    if( darkTimer )    // timer active
    {
      Tsecs--;                     // the darkTimer count
      
      if( lastTsecs != Tsecs )
      {
        lastTsecs = Tsecs;
        textN = "t0.txt=\"Timer: ";              // update banner
        textN.concat(Tsecs);
        textN.concat( "\"" );
        toNextion( textN );
      }

      if( Tsecs < 1 )
      {
        darkTimer = false;
        pOn = false;
        dOn = false;
        cOn = false;
        doStates();        
      }
    }
  }
}

