/*
   Do some testing of the Nextion display.

   Nextion serial to RX1/TX1
*/

#define dbug true

#include <DS3231.h>
#include <Wire.h>

#define redLED 13     // on board LED
#define startPin 9

DS3231 Clock;

bool Century = false;
bool h12;
bool PM;
byte ADay, AHour, AMinute, ASecond, ABits;
bool ADy, A12h, Apm;

byte year, month, date, DoW, hour, minute, second, lastsec,
     temperature, Tmax, Tmin;    // global time, temp
int mins2day;         // 1440 per day

byte dOn = false;
byte cOn = false;
byte pOn = false;
String textN;

//byte OK = false;
byte inbyte;
byte command = 0;

byte smsbuffer[16];


#include <LiquidCrystal_I2C.h>

LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);      // This works for 4x20
// LiquidCrystal_I2C lcd(0x3F, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);   // for MH board
// LiquidCrystal_I2C lcd(0x20, 4, 5, 6, 0, 1, 2, 3, 7, NEGATIVE);   // for mjkdz board


void toNextion( String  s )
{
  Serial1.print( s );
  Serial1.write(255);
  Serial1.write(255);
  Serial1.write(255);
}

void sendData()
{
  int h, m;

//  Serial1.println("AT + CMGS = \"" + destinationNumber + "\"");
  delay(1000);
  Serial1.print("Temp:");
  Serial1.print( temperature );

  Serial1.print(",");
  Serial1.print( Tmin );
  Serial1.print(",");
  Serial1.print( Tmax );

}


void txtNextion()
{
  textN = "t0.txt=\"";
  if ( dOn ) textN.concat("Din");
  if ( pOn ) textN.concat(" Por");
  if ( cOn ) textN.concat(" Cor");
  if ( !dOn && !pOn ) textN.concat("** 65 **");
  Serial1.print( textN );
  Serial1.print( "\"" );  
  Serial1.write(255);
  Serial1.write(255);
  Serial1.write(255);
}

void doStates()
{
  if ( dOn )
  {
    digitalWrite( redLED, HIGH );
    toNextion("r0.val=1");
    lcd.print("D on ");

    // and activate relays etc
  }
  else
  {
    digitalWrite( redLED, LOW );
    toNextion("r0.val=0");
    lcd.print("D off ");    
  }   

  if ( pOn )
  {
    toNextion("r1.val=1");
    lcd.print("P on ");
  }
  else
  {
    toNextion("r1.val=0");
    lcd.print("P off ");
  }   

  if ( cOn )          // linked to pOn
  {
    toNextion("r2.val=1");
  }
  else
  {
    toNextion("r2.val=0");
  }   

  txtNextion();
}


// Handle incoming nextion data

byte intosmsbuffer()
{
  byte button;
  inbyte = Serial1.read();

  // looking for e 0000 00 0x255 0x255 ox255   9ch
  //               bbbb 01    button 00 or 01

  smsbuffer[0] = smsbuffer[1];  // keep adding to end of array
  smsbuffer[1] = smsbuffer[2];
  smsbuffer[2] = smsbuffer[3];
  smsbuffer[3] = smsbuffer[4];
  smsbuffer[4] = smsbuffer[5];
  smsbuffer[5] = smsbuffer[6];
  smsbuffer[6] = inbyte;

  if ( dbug )
  {
    Serial.print("SMSbuffer=");
    Serial.print( smsbuffer[0] ); Serial.print(",");
    Serial.print( smsbuffer[1] ); Serial.print(",");
    Serial.print( smsbuffer[2] ); Serial.print(",");
    Serial.print( smsbuffer[3] ); Serial.print(",");
    Serial.print( smsbuffer[4] ); Serial.print(",");
    Serial.print( smsbuffer[5] ); Serial.print(",");
    Serial.println( smsbuffer[6] );
  }

  button = 0;
  
  if( (smsbuffer[0] == 101)       // "e"
        && (smsbuffer[4] == 255)
        && (smsbuffer[5] == 255)
        && (smsbuffer[6] == 255))
    {
      button = smsbuffer[2];
      // bstate = smsbuffer[3];

      if ( button==1)     // Dining
        if ( dOn )
        {
          dOn = false;
        }
        else
        {
          dOn = true;
        }   
      
      if ( button==2)   // Outside controls 2 lights
      {
        if ( pOn )
        {
          pOn = false;
        }
        else
        {
          pOn = true;
        }   
        
        if ( cOn )
        {
          cOn = false;
        }
        else
        {
          cOn = true;
        }   
      }

      if ( button==5 )
      {
        dOn = false;
        pOn = false;
        cOn = false;
      }
    }

    if(dbug) Serial.print("Button:"); Serial.println(button);

    if ( button > 0 )     // got one, action it and clear buffer
    {
      doStates();
      smsbuffer[0] = 0;
      smsbuffer[1] = 0;
      smsbuffer[2] = 0;
      smsbuffer[3] = 0;
      smsbuffer[4] = 0;
      smsbuffer[5] = 0;
      smsbuffer[6] = 0;
    }
}

void setup()
{
  Wire.begin();
  // initialize both serial ports:
  Serial.begin(115200);
  Serial1.begin(9600);         // Nextion

  toNextion("t0.txt=\"Ent V2\""); // set banner
  
  toNextion("r0.val=0");    // clear radio buttons
  toNextion("r1.val=0");
  toNextion("r2.val=0");

  toNextion("t1.txt=\"Di\"");   // override design names
  toNextion("t2.txt=\"Po\"");
  toNextion("t3.txt=\"Co\"");
  
  // settime();

  lcd.begin(20, 4);       // initialize the lcd for 20 chars 4 lines, turn on backlight
  lcd.setCursor(0, 0);

  pinMode( redLED, OUTPUT );
  digitalWrite( redLED, LOW );
  Tmin = 100; Tmax = 0;
}


void loop()
{
  if ( lastsec != second )      // one sec actions
  {
    lastsec = second; //lcdUpdate();

  }


  if ( Serial1.available() )      // put char into buffer to
    intosmsbuffer();
}

