/*
   Do some testing of the Nextion display.
   Tested on UNO now moved to 8266

   Nextion serial to RX1/TX1
*/

#define dbug true
#define buttonD 16      // dining lights
#define buttonP 14      // outside corner light
#define relayP 12       // corner light
#define relayD 13       // dining lights

byte dOn = false;
byte cOn = false;
byte pOn = false;
String textN;

byte inbyte;
byte command = 0;

byte smsbuffer[16];


void toNextion( String  s )
{
  Serial1.print( s );
  Serial1.write(255);
  Serial1.write(255);
  Serial1.write(255);
}



void txtNextion()
{
  textN = "t0.txt=\"";
  if ( dOn ) textN.concat("Din");
  if ( pOn ) textN.concat(" Por");
  if ( cOn ) textN.concat(" Cor");
  if ( !dOn && !pOn ) textN.concat("** 65 **");
  Serial1.print( textN );
  Serial1.print( "\"" );  
  Serial1.write(255);
  Serial1.write(255);
  Serial1.write(255);
}

void doStates()
{
  if ( dOn )
  {
    digitalWrite( LED_BUILTIN, HIGH );
    toNextion("r0.val=1");
    // and activate relays etc
  }
  else
  {
    digitalWrite( LED_BUILTIN, LOW );
    toNextion("r0.val=0");
  }   

  if ( pOn )
  {
    toNextion("r1.val=1");
  }
  else
  {
    toNextion("r1.val=0");
  }   

  if ( cOn )          // linked to pOn
  {
    toNextion("r2.val=1");
  }
  else
  {
    toNextion("r2.val=0");
  }   

  txtNextion();
}


// Handle incoming nextion data

byte intosmsbuffer()
{
  byte button;
  inbyte = Serial1.read();

  // looking for e 0000 00 0x255 0x255 ox255   9ch
  //               bbbb 01    button 00 or 01

  smsbuffer[0] = smsbuffer[1];  // keep adding to end of array
  smsbuffer[1] = smsbuffer[2];
  smsbuffer[2] = smsbuffer[3];
  smsbuffer[3] = smsbuffer[4];
  smsbuffer[4] = smsbuffer[5];
  smsbuffer[5] = smsbuffer[6];
  smsbuffer[6] = inbyte;

  if ( dbug )
  {
    Serial.print("SMSbuffer=");
    Serial.print( smsbuffer[0] ); Serial.print(",");
    Serial.print( smsbuffer[1] ); Serial.print(",");
    Serial.print( smsbuffer[2] ); Serial.print(",");
    Serial.print( smsbuffer[3] ); Serial.print(",");
    Serial.print( smsbuffer[4] ); Serial.print(",");
    Serial.print( smsbuffer[5] ); Serial.print(",");
    Serial.println( smsbuffer[6] );
  }

  button = 0;
  
  if( (smsbuffer[0] == 101)       // "e"
        && (smsbuffer[4] == 255)
        && (smsbuffer[5] == 255)
        && (smsbuffer[6] == 255))
    {
      button = smsbuffer[2];
      // bstate = smsbuffer[3];

      if ( button==1)     // Dining
        if ( dOn )
        {
          dOn = false;
        }
        else
        {
          dOn = true;
        }   
      
      if ( button==2)   // Outside controls 2 lights
      {
        if ( pOn )
        {
          pOn = false;
        }
        else
        {
          pOn = true;
        }   
        
        if ( cOn )
        {
          cOn = false;
        }
        else
        {
          cOn = true;
        }   
      }

      if ( button==5 )
      {
        dOn = false;
        pOn = false;
        cOn = false;
      }
    }

    if(dbug) Serial.print("Button:"); Serial.println(button);

    if ( button > 0 )     // got one, action it and clear buffer
    {
      //doStates();
      smsbuffer[0] = 0;
      smsbuffer[1] = 0;
      smsbuffer[2] = 0;
      smsbuffer[3] = 0;
      smsbuffer[4] = 0;
      smsbuffer[5] = 0;
      smsbuffer[6] = 0;
    }
}


void toggleP()
{
  if ( digitalRead( relayP ) == LOW )
  {
    Serial.println("P H");
    digitalWrite( relayP, HIGH );
  }
  else
  {
    Serial.println("P L");
    digitalWrite( relayP, LOW );    
  }
}


void toggleD()
{
  if ( digitalRead( relayD ) == LOW )
  {
    Serial.println("D H");
    digitalWrite( relayD, HIGH );
  }
  else
  {
    Serial.println("D L");
    digitalWrite( relayD, LOW );    
  }
}

void buttonHandler()    // takes 1 sec
{
  if ( digitalRead( buttonP ) == LOW ) toggleP();  
  if ( digitalRead( buttonD ) == LOW ) toggleD(); 
  
//  delay(500);

//  if ( digitalRead( buttonP ) == LOW &&
//    digitalRead( buttonD ) == LOW ) secs = secdel; // exit timer
}

void setup()
{

  pinMode( LED_BUILTIN, OUTPUT );
  pinMode( relayP, OUTPUT );
  digitalWrite( relayP, HIGH );   // active low relay
  pinMode( relayD, OUTPUT );
  digitalWrite( relayD, HIGH );   // active low relay
  pinMode( buttonP, INPUT );
  pinMode( buttonD, INPUT );

  Serial.begin(115200);
  Serial.println("Initializing... SLAVE");

 // Serial1.begin(9600);            // Nextion

  toNextion("t0.txt=\"Ent V2\""); // set banner
  
  toNextion("r0.val=0");          // clear radio buttons
  toNextion("r1.val=0");
  toNextion("r2.val=0");

  toNextion("t1.txt=\"Di\"");     // override design names
  toNextion("t2.txt=\"Po\"");
  toNextion("t3.txt=\"Co\"");
}


void loop()
{
  buttonHandler();

  if ( LED_BUILTIN == LOW )
    digitalWrite( LED_BUILTIN, HIGH );
  else
    digitalWrite( LED_BUILTIN, LOW );

  delay(500);

//  if ( Serial1.available() )      // put char into buffer to
//    intosmsbuffer();
}

