// Slave using Sonoff TH10  housing repaired with ESP-01 module
//
//  For controlling the corner outside light
//
// NB because GPIO2 must be high during power up we must put a 2k2 
// resistor from 2 to 3v3 otherwise it just hangs.

#define device '8'  // Device 8, corner light

// #include <Arduino.h>
#include <ESP8266WiFi.h>
extern "C" {
#include <espnow.h>
#include <user_interface.h>
}

#define WIFI_DEFAULT_CHANNEL 10

#define relay 2     // relay 
#define button 0    // to switch on sonoff

void printMacAddress(uint8_t* macaddr) 
{
  Serial.print("{");
  for (int i = 0; i < 6; i++) 
  {
    Serial.print("0x");
    Serial.print(macaddr[i], HEX);
    if (i < 5) Serial.print(',');
  }
  Serial.print("}");
}

void setup() 
{
  uint8_t macaddr[6];
  
  WiFi.disconnect();
  pinMode( LED_BUILTIN, OUTPUT );
  pinMode( relay, OUTPUT );
  digitalWrite( relay, LOW );

  pinMode( button, INPUT );

  Serial.begin(115200);
  Serial.println( "File: Enow-slave_sonTH_9_");
 Serial.println("Initializing... SLAVE");

  WiFi.mode(WIFI_AP_STA);
  wifi_get_macaddr(SOFTAP_IF, macaddr);
  Serial.print("mac address (SOFTAP_IF): "); printMacAddress(macaddr);
  Serial.println();

  if (esp_now_init() == 0) 
  {
    Serial.println("Now init OK");
  } 
  else 
  {
    Serial.println("Now init failed");
    ESP.restart();
    return;
  }

  Serial.println("ESP_NOW_ROLE_SLAVE");
  esp_now_set_self_role(ESP_NOW_ROLE_SLAVE);
  
  esp_now_register_recv_cb([](uint8_t *macaddr, uint8_t *data, uint8_t len) 
  {                 // this code runs when data received
    Serial.println("!!! we have a reading");
    Serial.print("recv_cb: mac address:  "); printMacAddress(macaddr);
    Serial.print("  Len:"); Serial.println(len);
    Serial.print("data: ");
    for (int i = 0; i < len; i++) 
    {
      //Serial.print(" 0x");
      //Serial.print(data[i], HEX);
      Serial.print(char(data[i]));
    }
    
    if ( data[0] == 'b' && data[1] == 'l' && data[2] == 'h' )     // it's legit
//      if ( data[3] == '8' || data[3] == '0' )                     // device 8 or ALL
      if ( data[3] == device || data[3] == '0' )                     // device 8 or ALL
        if ( data[4] == '0' ) 
        {
          digitalWrite( relay, LOW )  ;      // 1 = ON
          Serial.println("R Low");
        }
        else 
        {
          digitalWrite( relay, HIGH );
          Serial.println("R High");
        }
  
  });
}

void loop() 
{
  if ( digitalRead( button ) == LOW )
  {
    if ( digitalRead( relay ) == LOW )
      digitalWrite( relay, HIGH );
    else
      digitalWrite( relay, LOW );    
  }
  
  while( digitalRead( button ) == LOW ) yield();
  
  delay(1000);
}

